//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2018
//#
//#
//# File Name:  EPT_5M240_FG_S2_Top.v
//# Author:     Earth People Technology
//# Date:       December 12, 2018
//# Revision:   A
//#
//# Development: EPT Data Collection Project 
//# Application: Altera MAX V CPLD
//# Description: This file contains verilog code which will allow access
//#              to Active Transfer Library. 
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			12/12/18 	1			Created
//#
//#									
//#
//#######################################################################


//************************************************************************
//* Module Declaration
//************************************************************************

module EPT_5M240_AP_S2_Top (

    input wire                CLK_10MHZ,
    input wire                RST,	
	
	input wire                SW_USER_1,
	input wire                SW_USER_2,
	input wire                SW_USER_3,
	input wire                SW_USER_4,
	input wire                SW_USER_5,
	input wire                SW_USER_6,
	input wire                SW_USER_9,
	
	input wire                PWR_ENABLE,
	output wire               PWR_KILL,

	output reg  [10:0]        LED
	);

   //-----------------------------------------------
   // Parameters
   //-----------------------------------------------
   
   //LED State Machine Parameters
   parameter                   IDLE            = 0,
                               SELECT_MODE     = 1,
                               LOAD_REGISTER   = 2,
                               LOAD_LEDS       = 3,
                               WAIT_FOR_TIMER  = 4;
							   
   //Trigger Select State Machine
   parameter                   TRIGGER_IDLE      = 0,
                               TRIGGER_BTN_PRESS = 1,
                               TRIGGER_SELECT    = 2,
                               TRIGGER_RESET     = 3,
                               TRIGGER_COMPLETE  = 4;


   
   //Control Register selections for the LED Blinky
   parameter                   RANDOM_NUMBER   = 4'h1;
   parameter                   SHIFT_LEFT      = 4'h2;
   parameter                   SHIFT_RIGHT     = 4'h3;
   parameter                   COUNT_UP        = 4'h4;
   parameter                   COUNT_DOWN      = 4'h5;
   //parameter                   STATIC_VALUE    = 4'h6;
   
   //Timer Value Low Limit
`ifdef SIM
   parameter                   TIMER_START_VALUE = 24'h2710; //0x2710 = 1ms
   parameter                   TIMER_DEFAULT_LIMIT = 24'h2710; //0x2710 = 1ms
   parameter                   TIMER_STEP_SIZE = 24'h3e8; //0x3e8 = 0.1ms
`else
   //parameter                   TIMER_START_VALUE = 24'h4c4b40; //0x4c4b40 = 500ms
   parameter                   TIMER_START_VALUE = 24'h989680; //0x4c4b40 = 1,000ms
   parameter                   TIMER_DEFAULT_LIMIT = 24'h989680; //0x989680 = 1,000ms
   parameter                   TIMER_STEP_SIZE = 24'hc350; //0xc350 = 5ms
`endif

   
//***************************************************************************
//* Internal Signals and Registers Declarations
//***************************************************************************

	//Finite State Machine control registers
	reg  [4:0]                 state, next;

	//Trigger Byte
	reg  [7:0]                 trigger_in_byte;
	reg  [2:0]                 trigger_state;
	
	//control_register
	reg  [7:0]                 control_register;
	
	//Button registers
	wire [7:0]                 button_register;
	
	//LED registers
	//wire                       led_reset;		
	reg  [23:0]                timer_value;	
	//reg  [7:0]                 seed_value;
	reg  [23:0]                led_delay_counter;
	//reg  [10:0]                random_number;
	//reg  [10:0]                random_num_counter;
	reg  [10:0]                led_reg;
	//wire                       random_number_generated;
	reg                        start_blinky;
	reg  [10:0]                static_load_value;
	
	reg  [10:0]                shift_count_value;

`ifdef SIM
   reg [8*26:1] state_name;
`endif
	


//***************************************************************************
//* 	Signal Assignments	
//***************************************************************************

   assign    button_register = {!SW_USER_1, !SW_USER_2, !SW_USER_3, !SW_USER_4, !SW_USER_5, !SW_USER_6, !SW_USER_9, 1'b0};

   assign    PWR_KILL = 1'b0;
   
   //-----------------------------------------------
   // Trigger Byte Selection
   //
   // trigger_in_byte[0]       static_load_value <= transfer_in_byte;
   // trigger_in_byte[1]
   // trigger_in_byte[2]
   // trigger_in_byte[3]       shift_count_value <= transfer_in_byte[3:0];
   // trigger_in_byte[4]       timer_value <= timer_value - 1;
   // trigger_in_byte[5]       timer_value <= timer_value + 1;
   // trigger_in_byte[6]       Load Linear Feedback Shift Register
   // trigger_in_byte[7]       mode select
   //
   //trigger_state:
   //    TRIGGER_IDLE
   //    TRIGGER_BTN_PRESS
   //    TRIGGER_SELECT
   //    TRIGGER_RESET
   //    TRIGGER_COMPLETE
   //-----------------------------------------------

  always @(posedge CLK_10MHZ or negedge RST)
  begin
    if(!RST)
	begin
	     trigger_in_byte <= 8'b00100000;
		 trigger_state <= TRIGGER_IDLE;
	end
	else 
	begin
	   case(trigger_state)
	   TRIGGER_IDLE:
	      if(button_register > 1)
		     trigger_state <= TRIGGER_BTN_PRESS;
	   TRIGGER_BTN_PRESS:
	      trigger_state <= TRIGGER_SELECT;
	   TRIGGER_SELECT:
	   begin
	      if(!SW_USER_1 | !SW_USER_2 | !SW_USER_3 | !SW_USER_4)
	         trigger_in_byte[0] <= 1'b1;
		  else if(!SW_USER_5) //Increase Speed
		     trigger_in_byte[4] <= 1'b1;
		  else if(!SW_USER_6) //Decrease Speed
		     trigger_in_byte[5] <= 1'b1;
		  else if(!SW_USER_9) //Mode Selection
		     trigger_in_byte[7] <= 1'b1;
		  trigger_state <= TRIGGER_RESET; 
		end
		TRIGGER_RESET:
		begin
		   trigger_in_byte <= 0;
		      trigger_state <= TRIGGER_COMPLETE; 
	    end
	    TRIGGER_COMPLETE:
		begin
`ifdef SIM
		      trigger_state <= TRIGGER_IDLE; 
`else
		      trigger_state <= TRIGGER_IDLE; 
`endif
	    end
	   endcase
	end 
  end

   //-----------------------------------------------
   // Control Register Set
   //-----------------------------------------------

  always @(posedge CLK_10MHZ or negedge RST)
  begin
    if(!RST)
	     control_register[7:4] <= COUNT_UP;
	else 
	begin
	     if(!SW_USER_1)
		     control_register[7:4] <= SHIFT_LEFT;
	     else if(!SW_USER_2)
		     control_register[7:4] <= SHIFT_RIGHT;
	     else if(!SW_USER_3)
		     control_register[7:4] <= COUNT_UP;
	     else if(!SW_USER_4)
	         control_register[7:4] <= COUNT_DOWN;
	end 
  end

   
   //-----------------------------------------------
   // Load Timer Value
   //-----------------------------------------------

  always @(posedge CLK_10MHZ or negedge RST)
  begin
    if(!RST)
	     timer_value <= TIMER_START_VALUE; //24'h4c4b40; //TIMER_DEFAULT_LIMIT;
	else 
	begin
	     if(trigger_in_byte[4])
		 begin
		     if(timer_value > TIMER_STEP_SIZE)
	            timer_value <= timer_value - TIMER_STEP_SIZE;
		     else 
	            timer_value <= timer_value;
		 end
	     else if(trigger_in_byte[5])
		 begin
		     if(timer_value > (TIMER_DEFAULT_LIMIT + TIMER_STEP_SIZE))
			    timer_value <= timer_value;
		     else 
	            timer_value <= timer_value + TIMER_STEP_SIZE;
		 end
	end 
  end

   //-----------------------------------------------
   // Load Static Value
   //-----------------------------------------------

  always @(posedge CLK_10MHZ or negedge RST)
  begin
    if(!RST)
	     static_load_value <= 1;
	else 
	begin
	     if(trigger_in_byte[7])
	         static_load_value <= 1;
	end 
  end

   //-----------------------------------------------
   // LED Blinky start 
   //-----------------------------------------------
  always @(posedge CLK_10MHZ or negedge RST)
  begin
    if(!RST)
         start_blinky <= 1'b0;
	else 
	begin
	    if(control_register[7:4] > 0)
		begin
            start_blinky <= 1'b1;
		end
		else
            start_blinky <= 1'b0;		
 	end 
  end
 
   //-----------------------------------------------
   // LED Delay Timer Counter 
   //-----------------------------------------------
  always @(posedge CLK_10MHZ or negedge RST)
  begin
    if(!RST)
	     led_delay_counter <= TIMER_START_VALUE;
	else 
	begin
	     if(state[SELECT_MODE])
		 led_delay_counter <= timer_value;
		 else if(state[WAIT_FOR_TIMER])
	         led_delay_counter <= led_delay_counter - 1'd1;
	end 
  end

   //-----------------------------------------------
   // Random Number array
   //-----------------------------------------------
/*  always @(posedge CLK_10MHZ or negedge RST)
  begin
     if(!RST)
	 begin
	     random_number <= 4'h0;
		 random_num_counter <= 0;
	 end
	else 
	begin
	    if(random_num_counter < 4'h4 )
	    begin
	        random_num_counter <= random_num_counter + 1'd1;
			random_number[random_num_counter] <= random_number_generated;
		end
	    else 
		begin
	        random_num_counter <= 0;
			random_number[random_num_counter] <= random_number_generated;
		end
	end
  end
*/
   //-----------------------------------------------
   // Select the LED Function from the control
   // register bits 7 to 4   
   //-----------------------------------------------
  always @(posedge CLK_10MHZ or negedge RST)
  begin
    if(!RST)
	     led_reg <= 0;
	else 
	begin
        if (state[SELECT_MODE])
	    begin
	         case(control_register[7:4])
		       //RANDOM_NUMBER:
		       //   led_reg = random_number;
		       SHIFT_LEFT:
		          led_reg = shift_count_value;
		       SHIFT_RIGHT:
		          led_reg = shift_count_value;
		       COUNT_UP:
		          led_reg = shift_count_value;
		       COUNT_DOWN:
		          led_reg = shift_count_value;
		       //STATIC_VALUE:
		       //   led_reg = static_load_value;
		       default:
		          led_reg = 11'h0;
		      endcase 
        end	
    end
  end
  
   //-----------------------------------------------
   // Increment the Shift/Count value from the control
   // register bits 7 to 4   
   //-----------------------------------------------
  always @(posedge CLK_10MHZ or negedge RST)
  begin
    if(!RST)
	     shift_count_value <= 1;
	else 
	begin
	    if(trigger_in_byte[7])
		     shift_count_value <= static_load_value;
		else if (state[LOAD_REGISTER])
	    begin
	         case(control_register[7:4])
		     SHIFT_LEFT:
			 begin
		        shift_count_value[0] <= led_reg[10];
		        shift_count_value[10] <= led_reg[9];
		        shift_count_value[9] <= led_reg[8];
		        shift_count_value[8] <= led_reg[7];
		        shift_count_value[7] <= led_reg[6];
		        shift_count_value[6] <= led_reg[5];
		        shift_count_value[5] <= led_reg[4];
		        shift_count_value[4] <= led_reg[3];
		        shift_count_value[3] <= led_reg[2];
		        shift_count_value[2] <= led_reg[1];
		        shift_count_value[1] <= led_reg[0];
		     end
		     SHIFT_RIGHT:
			 begin
		        shift_count_value[0] <= led_reg[1];
		        shift_count_value[10] <= led_reg[0];
		        shift_count_value[9] <= led_reg[10];
		        shift_count_value[8] <= led_reg[9];
		        shift_count_value[7] <= led_reg[8];
		        shift_count_value[6] <= led_reg[7];
		        shift_count_value[5] <= led_reg[6];
		        shift_count_value[4] <= led_reg[5];
		        shift_count_value[3] <= led_reg[4];
		        shift_count_value[2] <= led_reg[3];
		        shift_count_value[1] <= led_reg[2];
		     end
		     COUNT_UP:
		        shift_count_value <= led_reg + 1'd1;
		     COUNT_DOWN:
		        shift_count_value <= led_reg - 1'd1;
		     default:
	            shift_count_value <= 0;
		    endcase   
         end
	end 
  end

   //-----------------------------------------------
   // Set the LED outputs 
   //-----------------------------------------------
  always @(posedge CLK_10MHZ or negedge RST)
  begin
    if(!RST)
	     LED <= 11'hz;
	else 
	begin
	  if(state[LOAD_LEDS])
	  begin
		if ( led_reg[0] )
			LED[0] = 1'b0;
		else 
			LED[0] = 1'bz;
		if ( led_reg[1] )
			LED[1] = 1'b0;
		else 
			LED[1] = 1'bz;
 		if ( led_reg[2] )
			LED[2] = 1'b0;
		else 
			LED[2] = 1'bz;
		if ( led_reg[3] )
			LED[3] = 1'b0;
		else 
			LED[3] = 1'bz;
		if ( led_reg[4] )
			LED[4] = 1'b0;
		else 
			LED[4] = 1'bz;
		if ( led_reg[5] )
			LED[5] = 1'b0;
		else 
			LED[5] = 1'bz;
		if ( led_reg[6] )
			LED[6] = 1'b0;
		else 
			LED[6] = 1'bz;
		if ( led_reg[7] )
			LED[7] = 1'b0;
		else 
			LED[7] = 1'bz;
		if ( led_reg[8] )
			LED[8] = 1'b0;
		else 
			LED[8] = 1'bz;
		if ( led_reg[9] )
			LED[9] = 1'b0;
		else 
			LED[9] = 1'bz;
		if ( led_reg[10] )
			LED[10] = 1'b0;
		else 
			LED[10] = 1'bz;
	  end
	end
  end
   //-----------------------------------------------
   // State Machine: Send Pattern to LEDs 
   //-----------------------------------------------
   // Next State Logic
   always @(posedge CLK_10MHZ or negedge RST)
     begin
	if (!RST)
	  begin
	     state <= 5'h00;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end

     // State Definitions
   always @ ( state or start_blinky or 
		      led_reg or led_delay_counter or control_register or
			  timer_value or shift_count_value) 
     begin
	     next = 5'h00;

	if (state[IDLE])
	  begin
	     if (start_blinky)
	       next[SELECT_MODE] = 1'b1;
	     else
	       next[IDLE] = 1'b1;
	  end
	
	if (state[SELECT_MODE])
	begin
       if(control_register[7:4] == 4'h0)        
            next[IDLE] = 1'b1;
         else		 
           next[LOAD_REGISTER] = 1'b1;
	end

	if (state[LOAD_REGISTER])
	begin
        next[LOAD_LEDS] = 1'b1;
    end
		
	if (state[LOAD_LEDS])
	begin
        next[WAIT_FOR_TIMER] = 1'b1;
    end
	
	if (state[WAIT_FOR_TIMER])
		if (led_delay_counter > 0)//<= timer_value)
			next[WAIT_FOR_TIMER] = 1'b1;
		else 
			next[SELECT_MODE] = 1'b1;

			
`ifdef SIM
 	   if ( state == ( 1 << IDLE ))
		   state_name = "IDLE";
	   else if ( state == ( 1 << SELECT_MODE ))
		   state_name = "SELECT_MODE";
	   else if ( state == ( 1 << LOAD_REGISTER ))
		   state_name = "LOAD_REGISTER";
	   else if ( state == ( 1 << LOAD_LEDS ))
		   state_name = "LOAD_LEDS";
	   else if ( state == ( 1 << WAIT_FOR_TIMER ))
		   state_name = "WAIT_FOR_TIMER";
`endif


	end//end of state machine

   
   //-----------------------------------------------
   // Linear Feedback Shift Register
   //-----------------------------------------------
/*  lfsr                         LFSR_INST
  (
   .Q                          (random_number_generated),
   .CLK                        (CLK_10MHZ),
   .RST                        (RST),
   .SEED                       (shift_count_value),
   .LOAD                       (shift_count_value)
   );	
*/	

endmodule
