//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2018
//#
//#
//# File Name:  EPT_5M240_FG_S2_Top.v
//# Author:     Earth People Technology
//# Date:       August 18, 2023
//# Revision:   A
//#
//# Development: EPT Data Collection Project 
//# Application: Altera MAX V CPLD
//# Description: This file contains verilog code which will allow access
//#              to Active Transfer Library. 
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			8/18/23 	1			Created
//#
//#									
//#
//#######################################################################


//************************************************************************
//* Module Declaration
//************************************************************************

module EPT_5M240_AP_S2_Top (

    input wire                CLK_10MHZ,
    input wire                RST_N,	
	
	input wire                SW_USER_1,
	input wire                SW_USER_2,
	input wire                SW_USER_3,
	input wire                SW_USER_4,
	input wire                SW_USER_5,
	input wire                SW_USER_6,
	input wire                SW_USER_7,
	
	output wire [4:0]         TEST_BUS,
	
    output wire               SPI_DATA_OUT,
    output wire               SPI_CLK,
	input wire                SPI_DATA_IN,
    output wire               SPI_CS_N,
	output wire               LATCH_DAC_OUT,


	
	output reg  [10:0]        LED
	);

   //-----------------------------------------------
   // Parameters
   //-----------------------------------------------
   
   //DAC Write State Machine Parameters
   parameter                   IDLE                = 0,
                               SELECT_VALUE        = 1,
                               LOAD_REGISTER       = 2,
                               LOAD_LEDS           = 3,
                               WRITE_DAC_UPPER     = 4,
							   WRITE_DAC_DELAY     = 5,
							   WRITE_DAC_LOWER     = 6,
							   COMPLETE_DAC_WRITE  = 7,
							   LATCH_DAC           = 8;
							   
   
//***************************************************************************
//* Internal Signals and Registers Declarations
//***************************************************************************

	//Finite State Machine control registers
	reg  [8:0]                 state, next;


    wire [6:0]                 user_input_select;
	  
	//FSM control registers
	reg                        start_dac_write;
	reg                        user_input_flag;

	//spi_iface_master control registers
	reg  [7:0]                 spi_master_din;

	reg                        spi_master_we_in;
	wire                       spi_master_rdy_in;
	wire [7:0]                 spi_master_dout;
	wire                       spi_master_we_out_n;
	reg                        spi_master_rdy_out;

	//LED Control Registers
	reg  [10:0]                led_select;

	//State[WRITE_TO_DAC} control registers
	reg  [5:0]                 dac_delay_counter;
	reg                        dac_upper_delay_flag;
	reg                        dac_lower_delay_flag;
	reg  [7:0]                 dac_lower_din;
	reg  [7:0]                 dac_upper_din;

	//SPI Bus control registers
    wire                      spi_miso;
    wire                      spi_clk;
	wire                      spi_mosi;
    wire                      spi_cs;
	
	//Latch DAC registers
	reg                        dac_latch_out;
	reg  [5:0]                 dac_latch_counter;
	reg                        dac_latch_flag;
	
	//DAC CS control
	reg                        dac_cs_out;
	reg                        dac_cs_state;

	
`ifdef SIM
   reg [8*26:1] state_name;
`endif
	


//***************************************************************************
//* 	Signal Assignments	
//***************************************************************************


   assign    PWR_KILL = 1'b0;
   assign    user_input_select = {!SW_USER_1, !SW_USER_2, !SW_USER_3, !SW_USER_4, !SW_USER_5, !SW_USER_6, !SW_USER_7};
   
   assign    SPI_DATA_OUT = spi_mosi;
   assign    spi_miso = SPI_DATA_IN;
   assign    SPI_CLK = spi_clk;
   assign    SPI_CS_N = dac_cs_out;
   assign    LATCH_DAC_OUT = 1'b0;//!dac_latch_out;
   
   //For Testing
   assign    TEST_BUS[0] = spi_clk;
   assign    TEST_BUS[1] = spi_mosi;
   assign    TEST_BUS[2] = spi_cs;
   assign    TEST_BUS[3] = !dac_latch_out;
   assign    TEST_BUS[4] = spi_clk;
 
   //-----------------------------------------------
   // Detect a pushbutton press to start the write
   // to the DAC
   //-----------------------------------------------
   always @(posedge CLK_10MHZ or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  start_dac_write <= 1'b0;
	  user_input_flag <= 1'b0;
	end
	else 
	  begin
	     if((user_input_select > 0 ) & !user_input_flag)
		 begin
	       start_dac_write <= 1'b1;
		   user_input_flag <= 1'b1;
		 end
	     else if((user_input_select > 0 ) & state[SELECT_VALUE]) 
		 begin
	       start_dac_write <= 1'b0;
		   user_input_flag <= 1'b1;
	     end
	     else if((user_input_select == 0 ) & user_input_flag) 
		 begin
	       start_dac_write <= 1'b0;
		   user_input_flag <= 1'b0;
	     end
	  end
     end // always @ (posedge CLK or negedge RST_N)   
 
   //-----------------------------------------------
   // Determine the value to write to the DAC
   // Must handle both dac_lower_din and dac_upper_din
   //-----------------------------------------------
   always @(posedge CLK_10MHZ or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  dac_upper_din <= 8'h78;
	  dac_lower_din <= 8'h00;
	  led_select <= 11'h00;
	end
	else 
	  begin
	     if (state[SELECT_VALUE])
		 begin
		    case(user_input_select)
		        7'b0000001:
				begin
		           dac_upper_din <= 8'h72; //Send 0x24 to DAC 
				   dac_lower_din <= 8'h40;
				   led_select <= 11'b00000000001; // LED[0], D8,  LED_GR_1_N, SW 1
				end
		        7'b0000010:
				begin
		           dac_upper_din <= 8'h74; //Send 0x48 to DAC
				   dac_lower_din <= 8'h80;
				   led_select <= 11'b00000001000; //LED[3], D7, LED_GR_4_N, SW 2
				end
		        7'b0000100:
				begin
		           dac_upper_din <= 8'h76; //Send 0x6c to DAC
				   dac_lower_din <= 8'hc0;
				   led_select <= 11'b00000000100; //LED[2], D6, LED_GR_3_N, SW 3
				end
		        7'b0001000:
				begin
		           dac_upper_din <= 8'h79; //Send 0x90 to DAC
				   dac_lower_din <= 8'h00;
				   led_select <= 11'b00000000010; //LED[1], D5, LED_GR_2_N, SW 4
				end
		        7'b0010000:
				begin
		           dac_upper_din <= 8'h7b; //Send 0xb4 to DAC
				   dac_lower_din <= 8'h40;
				   led_select <= 11'b00000100000;// LED[5], D9, LED_GR_6_N, SW 5
				end
		        7'b0100000:
				begin
		           dac_upper_din <= 8'h7d; //Send 0xd8 to DAC
				   dac_lower_din <= 8'h80;
				   led_select <= 11'b00010000000;// LED[7], D11, LED_GR_8_N, SW 6
				end
		        7'b1000000:
				begin
		           dac_upper_din <= 8'h7f; //Send 0xfc to DAC
				   dac_lower_din <= 8'hc0;
				   led_select <= 11'b01000000000; // LED[9], D14, LED_GR_10_N, SW 9
				end
		        default:
				begin
		           dac_upper_din <= 8'h78; //Send 0x80 to DAC
				   dac_lower_din <= 8'h00;
				   led_select <= 11'b00001000000;
				end
		 endcase
	  end
	  end
     end // always @ (posedge CLK or negedge RST_N)   

   //-----------------------------------------------
   // Set the selected LEDs on/off
   // 
   //-----------------------------------------------
   always @(posedge CLK_10MHZ or negedge RST_N)
   begin
	if (!RST_N)
	begin
	  LED <= 11'hzz;
	end
	else 
	  begin
         if (state[LOAD_LEDS])
		 begin
		    if ( led_select[0] )
			   LED[0] = 1'b0;
		    else 
			   LED[0] = 1'bz;
		    if ( led_select[1] )
			   LED[1] = 1'b0;
		    else 
			   LED[1] = 1'bz;
		    if ( led_select[2] )
			   LED[2] = 1'b0;
		    else 
			   LED[2] = 1'bz;
		    if ( led_select[3] )
			   LED[3] = 1'b0;
		    else 
			   LED[3] = 1'bz;
		    if ( led_select[4] )
			   LED[4] = 1'b0;
		    else 
			   LED[4] = 1'bz;
		    if ( led_select[5] )
			   LED[5] = 1'b0;
		    else 
			   LED[5] = 1'bz;
		    if ( led_select[6] )
			   LED[6] = 1'b0;
		    else 
			   LED[6] = 1'bz;
		    if ( led_select[7] )
			   LED[7] = 1'b0;
		    else 
			   LED[7] = 1'bz;
		    if ( led_select[8] )
			   LED[8] = 1'b0;
		    else 
			   LED[8] = 1'bz;
		    if ( led_select[9] )
			   LED[9] = 1'b0;
		    else 
			   LED[9] = 1'bz;
		    if ( led_select[10] )
			   LED[10] = 1'b0;
		    else 
			   LED[10] = 1'bz;
      end
	end 
	end

   //-----------------------------------------------
   // Assert the WE_IN pin on the SPI_IFACE_MASTER
   // starting the write to the DAC
   // Must handle both WRITE_DAC_UPPER   WRITE_DAC_LOWER
   //dac_lower_din dac_upper_din
   //-----------------------------------------------
   always @(posedge CLK_10MHZ or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  spi_master_din <= 0;
	  spi_master_we_in <= 1'b0;
	  dac_delay_counter <= 0;
	  dac_upper_delay_flag <= 1'b0;
	  dac_lower_delay_flag <= 1'b0;
	end
	else 
	  begin
	     if(state[WRITE_DAC_UPPER] & !dac_upper_delay_flag)
		 begin
		   spi_master_din <= dac_upper_din;
	       spi_master_we_in <= 1'b1;
		   dac_upper_delay_flag <= 1'b1;
		 end
	     else if(state[WRITE_DAC_UPPER] & dac_upper_delay_flag) 
		 begin
	       if(dac_delay_counter <= 22)
		   begin
		      dac_delay_counter <= dac_delay_counter + 1'd1;
		      if(dac_delay_counter > 6'h6)
		         spi_master_we_in <= 1'b0;
		   end
		   else
		   begin
		       dac_delay_counter <= 0;
		   end
	     end
		 else if(state[WRITE_DAC_DELAY])
         begin
            dac_delay_counter <= 0;
	        spi_master_we_in <= 1'b0;
            dac_upper_delay_flag <= 1'b0;
         end
	     else if(state[WRITE_DAC_LOWER] & !dac_lower_delay_flag)
		 begin
		   spi_master_din <= dac_lower_din;
	       spi_master_we_in <= 1'b1;
		   dac_lower_delay_flag <= 1'b1;
		   dac_delay_counter <= 0;
		 end
	     else if(state[WRITE_DAC_LOWER] & dac_lower_delay_flag) 
		 begin
	       if(dac_delay_counter <= 22)
		   begin
		      dac_delay_counter <= dac_delay_counter + 1'd1;
		      if(dac_delay_counter > 6'h6)
		         spi_master_we_in <= 1'b0;
		   end
		   else
		   begin
		       dac_delay_counter <= 0;
		   end
	     end
	     else if(state[COMPLETE_DAC_WRITE]) 
		 begin
	       spi_master_we_in <= 1'b0;
		   dac_delay_counter <= 0;
		   dac_upper_delay_flag <= 1'b0;
		   dac_lower_delay_flag <= 1'b0;
	     end
	  end
     end // always @ (posedge CLK or negedge RST_N)   
 
   //-----------------------------------------------
   // DAC Latch delay
   // 
   //-----------------------------------------------
   always @(posedge CLK_10MHZ or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  dac_latch_out <= 1'b0;
	  dac_latch_counter <= 0;
	  dac_latch_flag <= 1'b0;
	end
	else 
	  begin
	     if(state[LATCH_DAC] & !dac_latch_flag)
		 begin
	       dac_latch_out <= 1'b0;
		   dac_latch_flag <= 1'b1;
		 end
	     else if(state[LATCH_DAC] & dac_latch_flag) 
		 begin
	       if(dac_latch_counter < 6'h7)
		   begin
		      dac_latch_counter <= dac_latch_counter + 1'd1;
		      if(dac_latch_counter > 6'h5)
		         dac_latch_out <= 1'b1;
		   end
	     end
	     else if(state[IDLE]) 
		 begin
	       dac_latch_out <= 1'b0;
		   dac_latch_counter <= 0;
		   dac_latch_flag <= 1'b0;
	     end
	  end
     end // always @ (posedge CLK or negedge RST_N)   
 
   //-----------------------------------------------
   // DAC CS modification to allow the DAC_CS pin to
   // assert low for the entire two byte transfer
   // of the DAC Write. 
   //-----------------------------------------------
   always @(posedge CLK_10MHZ or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  dac_cs_out <= 1'b1;
	  dac_cs_state <= 2'b00;
	end
	else 
	  begin
	     if(!spi_cs & (dac_cs_state == 2'b00))
		 begin
	       dac_cs_out <= 1'b0;
		   dac_cs_state <= 2'b01;
		 end
	     else if(spi_cs & (dac_cs_state == 2'b01))
		 begin
	       dac_cs_out <= 1'b0;
		   dac_cs_state <= 2'b10;
		 end
	     else if(!spi_cs & (dac_cs_state == 2'b10))
		 begin
	       dac_cs_out <= 1'b0;
		   dac_cs_state <= 2'b11;
		 end
	     else if(spi_cs & (dac_cs_state == 2'b11))
		 begin
	       dac_cs_out <= 1'b1;
		   dac_cs_state <= 2'b00;
		 end
	     else if(state[IDLE]) 
		 begin
	       dac_cs_out <= 1'b1;
		   dac_cs_state <= 1'b0;
	     end
	  end
     end // always @ (posedge CLK or negedge RST_N)   
 


 
   //-----------------------------------------------
   // State Machine: Send 8 bit value to DAC 
   //-----------------------------------------------
   // Next State Logic
   always @(posedge CLK_10MHZ or negedge RST_N)
     begin
	if (!RST_N)
	  begin
	     state <= 9'h00;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end

     // State Definitions
   always @ ( state or start_dac_write or spi_master_rdy_in or
		      spi_master_we_in or dac_delay_counter or dac_latch_counter or
			  dac_upper_delay_flag or dac_lower_delay_flag) 
     begin
	     next = 9'h00;

	if (state[IDLE])
	  begin
	     if (start_dac_write)
	       next[SELECT_VALUE] = 1'b1;
	     else
	       next[IDLE] = 1'b1;
	  end
	
	if (state[SELECT_VALUE])
	begin
           next[LOAD_REGISTER] = 1'b1;
	end

	if (state[LOAD_REGISTER])
	begin
        next[LOAD_LEDS] = 1'b1;
    end
		
	if (state[LOAD_LEDS])
	begin
        next[WRITE_DAC_UPPER] = 1'b1;
    end
	
	if (state[WRITE_DAC_UPPER])
	begin
		if (dac_delay_counter > 21)//<= timer_value)
			next[WRITE_DAC_DELAY] = 1'b1;
		else 
			next[WRITE_DAC_UPPER] = 1'b1;
	end
			
	if (state[WRITE_DAC_DELAY])
	begin
        next[WRITE_DAC_LOWER] = 1'b1;
    end
		

	if (state[WRITE_DAC_LOWER])
	begin
		if (dac_delay_counter > 17)
			next[COMPLETE_DAC_WRITE] = 1'b1;
		else 
        next[WRITE_DAC_LOWER] = 1'b1;
    end

    if (state[COMPLETE_DAC_WRITE])
	begin
		if (spi_master_rdy_in)
			next[LATCH_DAC] = 1'b1;
		else 
        next[COMPLETE_DAC_WRITE] = 1'b1;
    end

	if (state[LATCH_DAC])
	begin
		if (dac_latch_counter > 6)
			next[IDLE] = 1'b1;
		else 
        next[LATCH_DAC] = 1'b1;
    end

			
`ifdef SIM
 	   if ( state == ( 1 << IDLE ))
		   state_name = "IDLE";
	   else if ( state == ( 1 << SELECT_VALUE ))
		   state_name = "SELECT_VALUE";
	   else if ( state == ( 1 << LOAD_REGISTER ))
		   state_name = "LOAD_REGISTER";
	   else if ( state == ( 1 << LOAD_LEDS ))
		   state_name = "LOAD_LEDS";
	   else if ( state == ( 1 << WRITE_DAC_UPPER  )) 
		   state_name = "WRITE_DAC_UPPER";
	   else if ( state == ( 1 << WRITE_DAC_DELAY  )) 
		   state_name = "WRITE_DAC_DELAY";
       else if ( state == ( 1 << WRITE_DAC_LOWER ))
		   state_name = "WRITE_DAC_LOWER";	   
	   else if ( state == ( 1 << COMPLETE_DAC_WRITE ))
		   state_name = "COMPLETE_DAC_WRITE";
	   else if ( state == ( 1 << LATCH_DAC ))
		   state_name = "LATCH_DAC";
	   
`endif


	end//end of state machine

   
   //-----------------------------------------------
   // DAC SPI Master Control
   //-----------------------------------------------
  spi_iface_master             DAC_SPI_INST
  (
   .CLK                        (CLK_10MHZ),
   .RST_N                      (RST_N),
   .CLK_100                    (),
   .RST_N_100                  (),
   
   .SPI_DATA_OUT               (spi_miso),
   .SPI_CLK                    (spi_clk),
   .SPI_DATA_IN                (spi_mosi),
   .SPI_CS_N                   (spi_cs),

   .DIN                        (spi_master_din), //SPI Master Write
   .WE_IN                      (spi_master_we_in),
   .RDY_IN                     (spi_master_rdy_in),

   .DOUT                      (spi_master_dout), //SPI Master Read
   .WE_OUT_N                  (spi_master_we_out_n),
   .RDY_OUT                   (spi_master_rdy_out)
 
   );	
	

endmodule
