//================================================================
//
//	Smartronix Proprietary Information
//
//
//	File Name:	dac_model.v
//	Author:	    R. Jolly
//	Date:		June 26, 2009
//	Revision:	A
//
//
//	Development:	Environment Control Module UAV
//	Application:	Altera Stratix II
//	Description:	This file contains Verilog code that emulates the Analog Devices AD7934.
//				
//				
//
//
//
//	Revision History:
//
//	DATE		VERSION		DETAILS
//	06/26/15		A		Created by R Jolly
//
//
//
//
//
//
//
//
//
//
//
//==================================================================

module dac_model(

		CLK,
		RESET,

		DAC_DATA,
		DAC_BUSY,  
		DAC_REFOUT,

		DAC_CSL,
		DAC_RDL,
		DAC_CONVST,

        DAC_CONVERSION_REG,
		DAC_SETUP_REG,
		DAC_AVERAGING_REG,
		DAC_RESET_REG

		);

//==================================================================
// Parameter Declarations
//==================================================================
    parameter   IDLE                    = 0,
                CONV_START              = 1,
                SET_BUSY_FLAG           = 2,
                WRITE_ADC_WORD          = 3,
                WRITE_ADC_WORD_DELAY    = 4,
                CS_DE_ASSERTED          = 5;

//==================================================================
// IO Port Declarations
//==================================================================
	// Clock 
    input                       CLK;

	// Reset
	input                       RESET; 

	// AD Stuff
	inout   [11:0]              AD_DATA;
	output                      AD_BUSY;
	input                       AD_CSL;
	input                       AD_RDL;
	input                       AD_CONVST;
	output                      AD_REFOUT;
	
	input   [7:0]               ADC_CONVERSION_REG;
	input   [7:0]               ADC_SETUP_REG;
	input   [7:0]               ADC_AVERAGING_REG;
	input   [7:0]               ADC_RESET_REG;



//==================================================================
// Register Declarations
//==================================================================
	wire	[15:0]				RECEIVE_REGISTER;

	wire	[11:0]				AD_DATA;
	reg						AD_BUSY;
  
  
	// State Machine Registers
	reg	[7:0]					state;
	reg	[7:0]					next;

	// ADC data registers Sensor board
	reg	[15:0]				adc_data_generator;	
	reg						adc_word_enable;

	// Busy flag counter
	reg	[15:0]				busy_flag_count;

	// Clock signals
	wire						CLK;

	// Byte enables
	wire	[5:0]					byte_enables;



//==================================================================
// Simulation Directives
//==================================================================

`ifdef SIM
	reg [8*30:1] state_name;
`endif

//=================================================================================
// Assignments	
//=================================================================================

	assign	AD_DATA = ( !AD_CSL & !AD_RDL ) ? adc_data_generator[11:0]: 12'hzzz;



//=================================================================================
// ADC Cycle: busy_flag_count 
//=================================================================================

	always @( negedge CLK or negedge RESET )
	begin
		if ( !RESET )
		begin
			busy_flag_count <= 16'h0;
		end
		else
		begin
        		if ( state[SET_BUSY_FLAG] )
			begin
				if( busy_flag_count < 16'hffff )
            			busy_flag_count <= busy_flag_count + 1;
			end
			else if ( state[WRITE_ADC_WORD] )
			begin
				busy_flag_count <= 8'h0;
			end
				
    		end
  end



//=================================================================================
// ADC Cycle: BUSY output register
//=================================================================================

	always @( negedge CLK or negedge RESET )
	begin
		if ( !RESET )
		begin
			AD_BUSY <= 1'b0;
		end
		else
		begin
        		if ( state[SET_BUSY_FLAG] )
			begin
					AD_BUSY <= 1'b1;
			end
			else if ( state[WRITE_ADC_WORD] )
			begin
 				AD_BUSY <= 1'b0;
			end
				
    		end
  end

//=================================================================================
//  ADC Cycle: adc_data_generator 
//=================================================================================

	always @( negedge CLK or negedge RESET )
	begin
		if ( !RESET )
		begin
      		adc_data_generator <= 16'h0;
    		end
  		else
    		begin
        		if ( state[SET_BUSY_FLAG] )
          		begin
              		adc_data_generator <= adc_data_generator + 1;
          		end
       		else if ( state[WRITE_ADC_WORD] )
            		adc_data_generator <= adc_data_generator;
    		end
	end




//=================================================================================
//	Sensor Board Model Finite State Machine
//	parameter	IDLE					= 0,
///			CONV_START				= 1,
//			SET_BUSY_FLAG			= 2,
//			WRITE_ADC_WORD			= 3,
//			CS_DE_ASSERTED			= 4;
//
//=================================================================================

	always @(  posedge CLK or negedge RESET  )
	begin
		if( !RESET )
		begin
			state <= 8'h00;
			state [IDLE] <= 1'b1;
		end
		else
			state <= next;
	end


	always @( state or AD_CONVST or busy_flag_count or adc_word_enable or 
          		AD_CSL or AD_RDL )
	begin
		next = 8'h00;
		if ( state[IDLE] )
		begin
			if (AD_CONVST)
				next[CONV_START] = 1'b1;

			else
				next[IDLE] = 1'b1;
		end

		if ( state[CONV_START] )
		begin
				next[SET_BUSY_FLAG] = 1'b1;
		end

		if ( state[SET_BUSY_FLAG] )
		begin
			if ( busy_flag_count >= 16'h01ff )
				next[WRITE_ADC_WORD] = 1'b1;
			else
				next[SET_BUSY_FLAG] = 1'b1;
		end

		if ( state[WRITE_ADC_WORD] )
		begin
			if ( !AD_CSL )
				next[WRITE_ADC_WORD_DELAY] = 1'b1;
			else
				next[WRITE_ADC_WORD] = 1'b1;
		end

		if ( state[WRITE_ADC_WORD_DELAY] )
		begin
			if ( !AD_RDL )
				next[CS_DE_ASSERTED] = 1'b1;
			else
				next[WRITE_ADC_WORD_DELAY] = 1'b1;
		end


		if ( state[CS_DE_ASSERTED] )
		begin
			if ( AD_CSL )
				next[IDLE] = 1'b1;
			else
				next[CS_DE_ASSERTED] = 1'b1;
		end

	end
//=================================================================
// For Simumlation, display the state names
//
//	parameter	IDLE					= 0,
///			CONV_START				= 1,
//			SET_BUSY_FLAG			= 2,
//			WRITE_ADC_WORD			= 3,
//			WRITE_ADC_WORD_DELAY		= 4,
//			CS_DE_ASSERTED			= 5;
//=================================================================

`ifdef SIM
always @( posedge CLK )
begin
	if ( state == (1 << IDLE))
		state_name = "IDLE";		
	if ( state == (1 << CONV_START))
		state_name = "CONV_START";		
	if ( state == (1 << SET_BUSY_FLAG))
		state_name = "SET_BUSY_FLAG";		
	if ( state == (1 << WRITE_ADC_WORD))
		state_name = "WRITE_ADC_WORD";		
	if ( state == (1 << WRITE_ADC_WORD_DELAY))
		state_name = "WRITE_ADC_WORD_DELAY";		
	if ( state == (1 << CS_DE_ASSERTED))
		state_name = "CS_DE_ASSERTED";		
end
`endif		


endmodule

