/******************************************************
 * Title: m25p40_flash_model.v 
 * Author: Richard Jolly                           
 * Creation Date: 6/26/2015                           
 * Description: This Module models the M25P40 Flash memory
 *
 * Copyright Earth People Technology, Inc. 2015
 *
******************************************************/
`include "../src/define.v"

`timescale 1 ns / 10 ps

module m25p40_flash_model
  (
	input wire                  CLK,
	input wire                  RST_N,

    // FLASH Interface	
	input wire                  FLASH_MOSI,
	input wire                  FLASH_CLK,
	output wire                 FLASH_MISO,
	input wire                  FLASH_CS,
	input wire                  FLASH_CNVST,	
	output wire                 FLASH_EOC

   );


   //----------------------------------------------
   // Parameter Declarations
   //----------------------------------------------
   parameter 	IDLE                        = 0,
                READ_MASTER                 = 1,
                ASSERT_TXE                  = 2,
                CHECK_FLASH_REGISTER          = 3,
				WAIT_FOR_EOC                = 4,
                ASSERT_FLASH_CONTROL          = 5,
                READ_FLASH                    = 6,
                ASSERT_USB_RXF              = 7,
                WAIT_FOR_USB_RD_N           = 8,
				WAIT_DATA_BYTE_COMPLETE     = 9,
                CHECK_DATA_XMIT_COMPLETE    = 10;
   
   parameter 	FLASH_EOC_DELAY_COUNT         = 16'h7f;
   //------------------------------------------------
   // Wire and Register Declarations                
   //------------------------------------------------
   	reg [10:0]                   state, next;


	//SPI Slave Registers
   	reg                         spi_slave_xmt_en;
   	reg                         spi_slave_rcv_rdy;
   	wire   [7:0]                spi_slave_data;     
   	wire                        spi_slave_xmt_rdy;
   	wire                        spi_slave_rcv_en;
	
	//FLASH Registers
   	wire   [9:0]                flash_data_out;
   	reg                         flash_cs;
   	reg                         flash_read;
	wire                        flash_busy;
	wire                        flash_conv_start;
	reg                         flash_end_of_conv;
	
	//Data Transfer registers
   	reg	[9:0]                   transmit_data_out;
	reg                         flash_data_xmit_msb;
	reg [7:0]                   flash_internal_register;
	
    // FLASH EOC Registers
	reg                         flash_eoc_reg;
   	reg	[15:0]                  flash_eoc_counter;
	
	//FLASH Internal Registers
   	reg	[7:0]                   flash_conversion_register;
   	reg	[7:0]                   flash_setup_register;	
   	reg	[7:0]                   flash_averaging_register;
   	reg	[7:0]                   flash_reset_register;	
	

    //TEST Output
	reg	[3:0]                   SPI_STATE;

      
`ifdef SIM
   	reg [8*30:1] state_name;
`endif
   
   //------------------------------------------------
   // Signal Assignments  
   //------------------------------------------------
   
   //SPI data register, bi-directional
   assign   spi_slave_data = (state[WAIT_FOR_USB_RD_N] & flash_data_xmit_msb) ? {6'h0,transmit_data_out[9:8]} : ((state[WAIT_FOR_USB_RD_N] & !flash_data_xmit_msb) ? transmit_data_out[7:0] : 10'hzzzz);
   //assign   FLASH_EOC = state[ASSERT_FLASH_CONTROL] ? 1'b1 : 1'b0;
   
   assign  flash_conv_start = ~FLASH_CNVST;
   assign  FLASH_EOC = ~flash_end_of_conv;

   //-----------------------------------------------
   // Create a State Test Output Signal
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  SPI_STATE <= 0;
	else
	  begin
	     if (state[IDLE])
            SPI_STATE <= 4'h0;
	     if (state[READ_MASTER])
	       SPI_STATE <= 4'h1;
	     if (state[ASSERT_TXE])
	       SPI_STATE <= 4'h2;
	     if (state[CHECK_FLASH_REGISTER])
	       SPI_STATE <= 4'h3;
	     if (state[WAIT_FOR_EOC])
		   SPI_STATE <= 4'h4;
		 if (state[ASSERT_FLASH_CONTROL])
	       SPI_STATE <= 4'h5;
	     if (state[READ_FLASH])
	       SPI_STATE <= 4'h6;
	     if (state[ASSERT_USB_RXF])
	       SPI_STATE <= 4'h7;
	     if (state[WAIT_FOR_USB_RD_N])
	       SPI_STATE <= 4'h8;
	     if (state[CHECK_DATA_XMIT_COMPLETE])
	       SPI_STATE <= 4'h9;

		  
	  end // else: !if(!RST_N)
     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Assert the FLASH_EOC signal to signal the FLASH
   // has completed the conversion and data is ready 
   // for transmission.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  		flash_end_of_conv <= 1'b0;
			flash_eoc_reg <= 1'b0;
			flash_eoc_counter <= 0;
	end
	else 
	  begin
	     if (state[ASSERT_FLASH_CONTROL] & !flash_eoc_reg)
		 begin
	  		 flash_end_of_conv <= 1'b1;
			 flash_eoc_reg <= 1'b1;
		 end
	     else if (flash_eoc_reg)
		 begin
		     if(flash_eoc_counter < flash_EOC_DELAY_COUNT)
	  		    flash_eoc_counter <= flash_eoc_counter + 1'd1;
			 else 
			 begin
	  		    flash_end_of_conv <= 1'b0;
			    flash_eoc_reg <= 1'b0;
			    flash_eoc_counter <= 0;
			 end
		 end
	  end
	end


   //-----------------------------------------------
   // Assert teh USB_RXF signal to start the transmit
   // of FLASH data over SPI.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  		spi_slave_xmt_en <= 1'b1;
	end
	else 
	  begin
	     if (state[ASSERT_USB_RXF])
		 begin
	  		 spi_slave_xmt_en <= 1'b0;
		 end
	     else if (state[WAIT_FOR_USB_RD_N])
		 begin
	  		 spi_slave_xmt_en <= 1'b1;
		 end
	  end
	end

   //-----------------------------------------------
   // Set the flash_data_xmit_msb to high when the initial
   // read of the FLASH occurs. This will cause the state
   // machine to transmit the MSB first. The state[CHECK_DATA_XMIT_COMPLETE]
   // will then swap the polarity of the flash_data_xmit_msb
   // and send the state machine to state[ASSERT_USB_RXF]
   // to transmit the LSB.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
			 flash_data_xmit_msb <= 1'b0;
	end
	else 
	  begin
	     if (state[READ_FLASH])
		 begin
			 flash_data_xmit_msb <= 1'b1;
		 end
	     if (state[CHECK_DATA_XMIT_COMPLETE])
		 begin
			 flash_data_xmit_msb <= 1'b0;
		 end
	  end
	end

   //-----------------------------------------------
   // Read the FLASH Data.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  		transmit_data_out <= 0;
	end
	else 
	  begin
	     if (state[READ_FLASH])
		 begin
	  		 transmit_data_out <= flash_data_out;
		 end
	  end
	end

	//-----------------------------------------------
   // Assert FLASH controls to read the FLASH Data.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  		flash_cs <= 1'b1;
		  	flash_read <= 1'b1;
	  		flash_averaging_register <= 0;
		  	flash_reset_register <= 0;
	end
	else 
	  begin
	     if (state[ASSERT_FLASH_CONTROL])
		 begin
	  		flash_cs <= 1'b0;
		  	flash_read <= 1'b0;
		 end
	     else if (state[ASSERT_USB_RXF])
		 begin
	  		flash_cs <= 1'b1;
		  	flash_read <= 1'b1;
		 end
	  end
	end

	  //-----------------------------------------------
   // Select an FLASH Register and send the contents
   // to the FLASH.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
   begin
	if (!RST_N)
	begin
	  		flash_conversion_register <= 0;
		  	flash_setup_register <= 0;
	  		flash_averaging_register <= 0;
		  	flash_reset_register <= 0;
	end
	else 
	  begin
	     if (state[CHECK_FLASH_REGISTER])
		 begin
	        casex(flash_internal_register[7:4])
			      4'b1xxx: //FLASH_INT_REG_CONV:
                  begin
				     flash_conversion_register = {1'b0,flash_internal_register[6:0]};
				  end
			      4'b01xx: //FLASH_INT_REG_SETUP:
			      begin
				     flash_setup_register = {2'b00,flash_internal_register[5:0]}; 
				  end
                  4'b001x: //FLASH_INT_REG_AVERAGING:
                  begin
				     flash_averaging_register = {3'b000,flash_internal_register[4:0]}; 
                  end
                  4'b0001: //FLASH_INT_REG_RESET:
                  begin
				     flash_reset_register = {4'b0000,flash_internal_register[3:0]};
                  end				  
			default:
			      begin
			       	flash_conversion_register <= 0;
		  	        flash_setup_register <= 0;
			       	flash_averaging_register <= 0;
			       	flash_reset_register <= 0;
				  end

			endcase
		 end
		end
	 end

   //-----------------------------------------------
   // Assert the USB_TXE signal to the SPI Slave Unit
   // 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  spi_slave_rcv_rdy <= 1'b0;
	else 
	  begin
	     if (state[ASSERT_TXE])
	       spi_slave_rcv_rdy <= 1'b1;
	     else if ( state[IDLE] ) 
	       spi_slave_rcv_rdy <= 1'b0;
	  end
     end // always @ (posedge CLK or negedge RST_N)   
 
	 
   //------------------------------------------------
   // Read the Data from SPI Slave unit
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  flash_internal_register <= 0;
	else
	  begin
	     if (state[READ_MASTER])
	       flash_internal_register <= spi_slave_data;
	  end
     end // always @ (posedge CLK or negedge RST_N)
	 

   //--------------------------------------------------
   // Write State Machine
   //--------------------------------------------------  
   // Next State Logic
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  begin
	     state <= 11'h000;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end


   always @ ( state or spi_slave_rcv_en or flash_busy
	      or spi_slave_rcv_rdy or flash_data_xmit_msb or spi_slave_xmt_rdy
	      or spi_slave_xmt_en or flash_conv_start or flash_cs or flash_read)

     begin
	next = 11'h000;

	if (state[IDLE])
	  begin
	     if(!spi_slave_rcv_en)
	       next[READ_MASTER] = 1'b1;
	     else if(flash_busy)
	       next[WAIT_FOR_EOC] = 1'b1;
		else
	       next[IDLE] = 1'b1;			
	  end

	if (state[READ_MASTER])
	  next[ASSERT_TXE] = 1'b1;

	if (state[ASSERT_TXE])
	  begin
		if(spi_slave_rcv_rdy)
			next[CHECK_FLASH_REGISTER] = 1'b1;
	     else
			next[ASSERT_TXE] = 1'b1;
	  end

	if (state[CHECK_FLASH_REGISTER])
		    next[IDLE] = 1'b1;

	if (state[WAIT_FOR_EOC])
	begin
	    if(!flash_busy)
		    next[ASSERT_FLASH_CONTROL] = 1'b1;
	    else 
		    next[WAIT_FOR_EOC] = 1'b1;
	end

	if (state[ASSERT_FLASH_CONTROL])
		    next[READ_FLASH] = 1'b1;

	if (state[READ_FLASH])
	  next[ASSERT_USB_RXF] = 1'b1;

	if (state[ASSERT_USB_RXF])
	       next[WAIT_FOR_USB_RD_N] = 1'b1;

	if (state[WAIT_FOR_USB_RD_N])
	  begin
	  	if(!spi_slave_xmt_rdy)
			next[WAIT_DATA_BYTE_COMPLETE] = 1'b1;
		else
			next[WAIT_FOR_USB_RD_N] = 1'b1;			
	  end 

	if (state[WAIT_DATA_BYTE_COMPLETE])
	begin
		if(spi_slave_xmt_rdy)
			next[CHECK_DATA_XMIT_COMPLETE] = 1'b1;			
		else 
			next[WAIT_DATA_BYTE_COMPLETE] = 1'b1;	
	end

	if (state[CHECK_DATA_XMIT_COMPLETE])
	begin
        if(flash_data_xmit_msb)
		   next[ASSERT_USB_RXF] = 1'b1;
		else 
			next[IDLE] = 1'b1;	
	end

	
`ifdef SIM
	if (state == (1 << IDLE))
	  state_name = "IDLE";
	else if (state == (1 << READ_MASTER))
	  state_name = "READ_MASTER";
	else if (state == (1 << ASSERT_TXE))
	  state_name = "ASSERT_TXE";
	else if (state == (1 << CHECK_FLASH_REGISTER))
	  state_name = "CHECK_FLASH_REGISTER";
	else if (state == (1 << WAIT_FOR_EOC))
	  state_name = "WAIT_FOR_EOC";
	else if (state == (1 << ASSERT_FLASH_CONTROL))
	  state_name = "ASSERT_FLASH_CONTROL";
	else if (state == (1 << READ_FLASH))
	  state_name = "READ_FLASH";
	else if (state == (1 << ASSERT_USB_RXF))
	  state_name = "ASSERT_USB_RXF";
	else if (state == (1 << WAIT_FOR_USB_RD_N))
	  state_name = "WAIT_FOR_USB_RD_N";
	else if (state == (1 << WAIT_DATA_BYTE_COMPLETE))
	  state_name = "WAIT_DATA_BYTE_COMPLETE";
	else if (state == (1 << CHECK_DATA_XMIT_COMPLETE))
	  state_name = "CHECK_DATA_XMIT_COMPLETE";

	  
`endif
     end // always @ (...

   //----------------------------------------------
   // Instantiate the Leaf FSM Modules
   //----------------------------------------------
   spi_tiny_slave_model         SPI_TINY_SLAVE_MODEL_INST
     (
      .CLK                      (CLK), 
      .RST_N                    (RST_N),
      
      .SPI_DATA_OUT             (FLASH_MISO),
      .SPI_CLK                  (FLASH_CLK),
      .SPI_DATA_IN              (FLASH_MOSI),
      .SPI_CS_N                 (FLASH_CS),

	  .USB_RXF_N                (spi_slave_xmt_en),
      .USB_TXE_N                (spi_slave_rcv_rdy),
	  .USB_DATA                 (spi_slave_data),
      
	  .USB_RD_N			        (spi_slave_xmt_rdy),
	  .USB_WR                   (spi_slave_rcv_en),
	  .USB_SI                   ()
      );

   flash_model                    FLASH_MODEL_INST
     (
      .CLK                      (CLK), 
      .RESET                    (RST_N),

      .DAC_DATA                  (flash_data_out),
      .DAC_BUSY                  (flash_busy),  
      .DAC_REFOUT                (),

      .DAC_CSL                   (flash_cs),
      .DAC_RDL                   (flash_read),
      .DAC_CONVST                (flash_conv_start),
	  
	  .DAC_CONVERSION_REG       (flash_conversion_register),
	  .DAC_SETUP_REG            (flash_setup_register),
	  .DAC_AVERAGING_REG        (flash_averaging_register),
	  .DAC_RESET_REG            (flash_reset_register)

      );

endmodule // audio_SPI_fsm

   