/******************************************************
 * Title: mcp4901_dac_model.v 
 * Author: Richard Jolly                           
 * Creation Date: 6/26/2015                           
 * Description: This Module is models the MCP4901 1 Channel
 * DAC. It is a SPI Slave device that has a speed of 300KSa/sec
 *
 * Copyright Earth People Technology, Inc. 2015
 *
******************************************************/
`include "../src/define.v"

`timescale 1 ns / 10 ps

module mcp4901_dac_model
  (
	input wire                  CLK,
	input wire                  RST_N,

    // DAC Interface	
	input wire                  DAC_MOSI,
	input wire                  DAC_CLK,
	output wire                 DAC_MISO,
	input wire                  DAC_CS,
	input wire                  DAC_CNVST,	
	output wire                 DAC_EOC

   );


   //----------------------------------------------
   // Parameter Declarations
   //----------------------------------------------
   parameter 	IDLE                        = 0,
                READ_FIRST_BYTE             = 1,
                READ_SECOND_BYTE            = 2,
                UPDATE_DAC_REGISTER         = 3,
                DAC_WRITE_COMPLETE          = 4;
   
   parameter 	DAC_EOC_DELAY_COUNT         = 16'h7f;
   //------------------------------------------------
   // Wire and Register Declarations                
   //------------------------------------------------
   	reg [4:0]                   state, next;


	//SPI Slave Registers
   	reg                         spi_slave_xmt_en;
   	reg                         spi_slave_rcv_rdy;
   	wire   [7:0]                spi_slave_data;     
   	wire                        spi_slave_xmt_rdy;
   	wire                        spi_slave_rcv_en;
	
	//DAC Registers
   	wire   [9:0]                dac_data_out;
   	reg                         dac_cs;
   	reg                         dac_read;
	wire                        dac_busy;
	wire                        dac_conv_start;
	reg                         dac_end_of_conv;
	
	//Data Transfer registers
   	reg	[9:0]                   transmit_data_out;
	reg                         dac_data_xmit_msb;
	reg [7:0]                   dac_internal_register;
	
    // DAC EOC Registers
	reg                         dac_eoc_reg;
   	reg	[15:0]                  dac_eoc_counter;
	
	//DAC Internal Registers
   	reg	[7:0]                   dac_conversion_register;
   	reg	[7:0]                   dac_setup_register;	
   	reg	[7:0]                   dac_averaging_register;
   	reg	[7:0]                   dac_reset_register;	
	

    //TEST Output
	reg	[3:0]                   SPI_STATE;

      
`ifdef SIM
   	reg [8*30:1] state_name;
`endif
   
   //------------------------------------------------
   // Signal Assignments  
   //------------------------------------------------
   
   //SPI data register, bi-directional
   assign   spi_slave_data = (state[WAIT_FOR_USB_RD_N] & dac_data_xmit_msb) ? {6'h0,transmit_data_out[9:8]} : ((state[WAIT_FOR_USB_RD_N] & !dac_data_xmit_msb) ? transmit_data_out[7:0] : 10'hzzzz);
   //assign   DAC_EOC = state[ASSERT_DAC_CONTROL] ? 1'b1 : 1'b0;
   
   assign  dac_conv_start = ~DAC_CNVST;
   assign  DAC_EOC = ~dac_end_of_conv;

   //-----------------------------------------------
   // Create a State Test Output Signal
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  SPI_STATE <= 0;
	else
	  begin
	     if (state[IDLE])
            SPI_STATE <= 4'h0;
	     if (state[READ_FIRST_BYTE])
	       SPI_STATE <= 4'h1;
	     if (state[READ_SECOND_BYTE])
	       SPI_STATE <= 4'h2;
	     if (state[UPDATE_DAC_REGISTER])
	       SPI_STATE <= 4'h3;
	     if (state[DAC_WRITE_COMPLETE])
		   SPI_STATE <= 4'h4;

		  
	  end // else: !if(!RST_N)
     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Assert the DAC_EOC signal to signal the DAC
   // has completed the conversion and data is ready 
   // for transmission.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  		dac_end_of_conv <= 1'b0;
			dac_eoc_reg <= 1'b0;
			dac_eoc_counter <= 0;
	end
	else 
	  begin
	     if (state[ASSERT_DAC_CONTROL] & !dac_eoc_reg)
		 begin
	  		 dac_end_of_conv <= 1'b1;
			 dac_eoc_reg <= 1'b1;
		 end
	     else if (dac_eoc_reg)
		 begin
		     if(dac_eoc_counter < dac_EOC_DELAY_COUNT)
	  		    dac_eoc_counter <= dac_eoc_counter + 1'd1;
			 else 
			 begin
	  		    dac_end_of_conv <= 1'b0;
			    dac_eoc_reg <= 1'b0;
			    dac_eoc_counter <= 0;
			 end
		 end
	  end
	end


   //-----------------------------------------------
   // Assert teh USB_RXF signal to start the transmit
   // of DAC data over SPI.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  		spi_slave_xmt_en <= 1'b1;
	end
	else 
	  begin
	     if (state[ASSERT_USB_RXF])
		 begin
	  		 spi_slave_xmt_en <= 1'b0;
		 end
	     else if (state[WAIT_FOR_USB_RD_N])
		 begin
	  		 spi_slave_xmt_en <= 1'b1;
		 end
	  end
	end

   //-----------------------------------------------
   // Set the dac_data_xmit_msb to high when the initial
   // read of the DAC occurs. This will cause the state
   // machine to transmit the MSB first. The state[CHECK_DATA_XMIT_COMPLETE]
   // will then swap the polarity of the dac_data_xmit_msb
   // and send the state machine to state[ASSERT_USB_RXF]
   // to transmit the LSB.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
			 dac_data_xmit_msb <= 1'b0;
	end
	else 
	  begin
	     if (state[READ_DAC])
		 begin
			 dac_data_xmit_msb <= 1'b1;
		 end
	     if (state[CHECK_DATA_XMIT_COMPLETE])
		 begin
			 dac_data_xmit_msb <= 1'b0;
		 end
	  end
	end

   //-----------------------------------------------
   // Read the DAC Data.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  		transmit_data_out <= 0;
	end
	else 
	  begin
	     if (state[READ_DAC])
		 begin
	  		 transmit_data_out <= dac_data_out;
		 end
	  end
	end

	//-----------------------------------------------
   // Assert DAC controls to read the DAC Data.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  		dac_cs <= 1'b1;
		  	dac_read <= 1'b1;
	  		dac_averaging_register <= 0;
		  	dac_reset_register <= 0;
	end
	else 
	  begin
	     if (state[ASSERT_DAC_CONTROL])
		 begin
	  		dac_cs <= 1'b0;
		  	dac_read <= 1'b0;
		 end
	     else if (state[ASSERT_USB_RXF])
		 begin
	  		dac_cs <= 1'b1;
		  	dac_read <= 1'b1;
		 end
	  end
	end

	  //-----------------------------------------------
   // Select an DAC Register and send the contents
   // to the DAC.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
   begin
	if (!RST_N)
	begin
	  		dac_conversion_register <= 0;
		  	dac_setup_register <= 0;
	  		dac_averaging_register <= 0;
		  	dac_reset_register <= 0;
	end
	else 
	  begin
	     if (state[CHECK_DAC_REGISTER])
		 begin
	        casex(dac_internal_register[7:4])
			      4'b1xxx: //DAC_INT_REG_CONV:
                  begin
				     dac_conversion_register = {1'b0,dac_internal_register[6:0]};
				  end
			      4'b01xx: //DAC_INT_REG_SETUP:
			      begin
				     dac_setup_register = {2'b00,dac_internal_register[5:0]}; 
				  end
                  4'b001x: //DAC_INT_REG_AVERAGING:
                  begin
				     dac_averaging_register = {3'b000,dac_internal_register[4:0]}; 
                  end
                  4'b0001: //DAC_INT_REG_RESET:
                  begin
				     dac_reset_register = {4'b0000,dac_internal_register[3:0]};
                  end				  
			default:
			      begin
			       	dac_conversion_register <= 0;
		  	        dac_setup_register <= 0;
			       	dac_averaging_register <= 0;
			       	dac_reset_register <= 0;
				  end

			endcase
		 end
		end
	 end

   //-----------------------------------------------
   // Assert the USB_TXE signal to the SPI Slave Unit
   // 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  spi_slave_rcv_rdy <= 1'b0;
	else 
	  begin
	     if (state[ASSERT_TXE])
	       spi_slave_rcv_rdy <= 1'b1;
	     else if ( state[IDLE] ) 
	       spi_slave_rcv_rdy <= 1'b0;
	  end
     end // always @ (posedge CLK or negedge RST_N)   
 
	 
   //------------------------------------------------
   // Read the Data from SPI Slave unit
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  dac_internal_register <= 0;
	else
	  begin
	     if (state[READ_MASTER])
	       dac_internal_register <= spi_slave_data;
	  end
     end // always @ (posedge CLK or negedge RST_N)
	 

   //--------------------------------------------------
   // Write State Machine
   //--------------------------------------------------  
   // Next State Logic
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  begin
	     state <= 5'h000;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end


   always @ ( state or spi_slave_rcv_en or dac_busy
	      or spi_slave_rcv_rdy or dac_data_xmit_msb or spi_slave_xmt_rdy
	      or spi_slave_xmt_en or dac_conv_start or dac_cs or dac_read)

     begin
	next = 5'h000;

	if (state[IDLE])
	  begin
	     if(!spi_slave_rcv_en)
	       next[READ_FIRST_BYTE] = 1'b1;
		else
	       next[IDLE] = 1'b1;			
	  end

	if (state[READ_FIRST_BYTE])
	  begin
		if(spi_slave_rcv_rdy)
			next[READ_SECOND_BYTE] = 1'b1;
	     else
			next[READ_FIRST_BYTE] = 1'b1;
	  end

	if (state[READ_SECOND_BYTE])
	begin
	    if(!dac_busy)
		    next[UPDATE_DAC_REGISTER] = 1'b1;
	    else 
		    next[READ_SECOND_BYTE] = 1'b1;
	end

	if (state[UPDATE_DAC_REGISTER])
	  begin
	  	if(!spi_slave_xmt_rdy)
			next[DAC_WRITE_COMPLETE] = 1'b1;
		else
			next[UPDATE_DAC_REGISTER] = 1'b1;			
	  end 

	if (state[DAC_WRITE_COMPLETE])
	begin
        if(dac_data_xmit_msb)
		   next[DAC_WRITE_COMPLETE] = 1'b1;
		else 
			next[IDLE] = 1'b1;	
	end

	
`ifdef SIM
	if (state == (1 << IDLE))
	  state_name = "IDLE";
	else if (state == (1 << READ_FIRST_BYTE))
	  state_name = "READ_FIRST_BYTE";
	else if (state == (1 << READ_SECOND_BYTE))
	  state_name = "READ_SECOND_BYTE";
	else if (state == (1 << UPDATE_DAC_REGISTER))
	  state_name = "UPDATE_DAC_REGISTER";
	else if (state == (1 << DAC_WRITE_COMPLETE))
	  state_name = "DAC_WRITE_COMPLETE";

	  
`endif
     end // always @ (...

   //----------------------------------------------
   // Instantiate the Leaf FSM Modules
   //----------------------------------------------
   spi_tiny_slave_model         SPI_TINY_SLAVE_MODEL_INST
     (
      .CLK                      (CLK), 
      .RST_N                    (RST_N),
      
      .SPI_DATA_OUT             (DAC_MISO),
      .SPI_CLK                  (DAC_CLK),
      .SPI_DATA_IN              (DAC_MOSI),
      .SPI_CS_N                 (DAC_CS),

	  .USB_RXF_N                (spi_slave_xmt_en),
      .USB_TXE_N                (spi_slave_rcv_rdy),
	  .USB_DATA                 (spi_slave_data),
      
	  .USB_RD_N			        (spi_slave_xmt_rdy),
	  .USB_WR                   (spi_slave_rcv_en),
	  .USB_SI                   ()
      );

   dac_model                    DAC_MODEL_INST
     (
      .CLK                      (CLK), 
      .RESET                    (RST_N),

      .DAC_DATA                  (dac_data_out),
      .DAC_BUSY                  (dac_busy),  
      .DAC_REFOUT                (),

      .DAC_CSL                   (dac_cs),
      .DAC_RDL                   (dac_read),
      .DAC_CONVST                (dac_conv_start),
	  
	  .DAC_CONVERSION_REG       (dac_conversion_register),
	  .DAC_SETUP_REG            (dac_setup_register),
	  .DAC_AVERAGING_REG        (dac_averaging_register),
	  .DAC_RESET_REG            (dac_reset_register)

      );

endmodule // audio_SPI_fsm

   